# -*- coding: utf-8 -*-
## (c) 2010 Sébastien L. W. Baelde
## Contact : clic4@clic4.org

import pygame, math, datetime, sys

class Vecteur2D:
	""" Mémorisation et opérations sur un point en 2D. """
	def __init__(self, x, y):
		"""	"""
		self.x = float(x)
		self.y = float(y)
		
	def __str__(self):
		return "x: "+str(self.x)+" y: "+str(self.y)
		
	def __add__(self, v):
		return Vecteur2D(v.x+self.x, v.y+self.y)

	def __mul__(self, v):
		return Vecteur2D(self.x*v.x, self.y*v.y)

	def cloner(self):
		""" Retourne une copie du vecteur. """
		return Vecteur2D( self.x, self.y)

	def multScalaire(self, scalaire):
		self.x *= scalaire
		self.y *= scalaire

class Vecteur3D():
	""" Mémorisation et opérations sur un point en 3D. """
	def __init__(self, x, y, z):
		self.x, self.y, self.z = x, y, z
		
	def cloner(self):
		return Vecteur3D(self.x, self.y, self.z)

class Matrice3D:
	""" Matrice pour effectuer des transformations dans l'espace 3D. """
	def __init__(self):
		pass
	
	def rotation (self, rotX, rotY, rotZ):
		""" Matrice de rotation autour des axes x, y, z. """
		rad = math.pi/180.0
		sx, cx, sy = math.sin(rad * rotX), math.cos(rad * rotX), math.sin(rad * rotY)
		cy, sz, cz = math.cos(rad * rotY), math.sin(rad * rotZ), math.cos(rad * rotZ)
		matRot = [  [cy*cz,          cy*sz,          -sy, 0],
					[sx*sy*cz-cx*sz, sx*sy*sz+cx*cz, sx*cy, 0],
					[cx*sy*cz+sx*sz, cx*sy*sz-sx*cz, cx*cy, 0],
					[0, 0, 0, 1]
				]
		return matRot	

	def multiplierVecteur(self, vecteur, matrice):
		""" Multiplication d'un Vecteur3D avec une matrice. """
		vec = vecteur.cloner()
		vecRes = Vecteur3D(vec.x * matrice[0][0] + vec.y * matrice[1][0] + vec.z * matrice[2][0] + matrice[3][0],
						   vec.x * matrice[0][1] + vec.y * matrice[1][1] + vec.z * matrice[2][1] + matrice[3][1],
						   vec.x * matrice[0][2] + vec.y * matrice[1][2] + vec.z * matrice[2][2] + matrice[3][2])

		return vecRes

class Moteur2D:
	""" Boucle principale du jeu. Gestion de la logique et de l'affichage. """
	def __init__(self):
		self.dureeRound = 50  # Durée d'un round en ms
		self.jeuActif = True
		self.tLogique = 0

	def maj (self):
		""" Mise à jour du programme """
		t0 = pygame.time.get_ticks()
		while(self.jeuActif):
			t1 = pygame.time.get_ticks()
			if( (t1-t0) > self.dureeRound ):
				# Logique
				self.majLogique()
				t0 += self.dureeRound
			if ( (t1-t0) > self.dureeRound ):
				t0 = t1 - self.dureeRound
				self.tLogique = t1-t0
			# Affichage
			self.majAffichage()

	def majLogique (self):
		""" Màj de la section logique. """
		pass

	def majAffichage(self):
		""" Màj de l'affichage. """
		pass

class Boule:
	""" Représente et gère une pseudo sphère pour l'affichage des heures et minutes """
	def __init__(self, pos, rayon, prof, focale, dimEcran, img=None, img_i=None):
		self.pos = pos.cloner()   # Position
		self.prof = prof          # Profondeur d'affichage
		self.focale = focale
		self.dimEcran = dimEcran.cloner()
		self.couleurTexte = (255, 255, 255)
		self.img = img            # Image par défaut
		self.img_i = img_i        # Image pour l'effet illuminé.
		self.rayon = rayon
		self.masquer = False      # Si vrai, boule masquée
		self.illuminer = False    # Si vrai, effet d'illumination sur la boule
		
		# Texte
		self.tailleTexte = 30
		self.font = pygame.font.Font("Perfect Dark Zero.ttf", self.tailleTexte)
		self.texte = None
		
	def afficher(self, surface):
		""" Affichage de la Boule sur surface. """
		pers = float(self.focale) / float(self.pos.z+self.focale)
		posProj = Vecteur2D(self.pos.x, self.pos.y)
		posProj.multScalaire(pers)
		demiEcran = self.dimEcran * Vecteur2D(0.5, 0.5)
		posProj += demiEcran
		rayon = self.rayon * pers
		if rayon < 0:
			rayon = 0.0
		
		if not self.masquer:		
			if self.illuminer:
				surfaceBoule = pygame.transform.scale(self.img_i, ( int(rayon*pers), int(rayon*pers) ) )
			else:
				surfaceBoule = pygame.transform.scale(self.img, ( int(rayon*pers), int(rayon*pers) ) )
			# Ajustement de la valeur alhpa de la Boule selon la distance z
			surfaceBoule = surfaceBoule.convert()
			surfaceBoule.set_colorkey((0, 0, 0))
			surfaceBoule.set_alpha( 255.0 - self.pos.z*2 )

			# Affichage du texte
			if self.texte != None:
				self.font = pygame.font.Font("Perfect Dark Zero.ttf", int(self.tailleTexte*pers))
				txt = self.font.render(self.texte, True, self.couleurTexte)
				dimTxt = float(txt.get_width())
				surfaceBoule.blit(txt, ((surfaceBoule.get_width()/2.0)-dimTxt/2.0, surfaceBoule.get_height()/2.0-1.5*(self.tailleTexte*pers)/2.0 ))

			surface.blit(surfaceBoule, (posProj.x-rayon, posProj.y-rayon) )

class Horloge(Moteur2D):
	""" Classe Horloge. """
	def __init__(self, surface):
		Moteur2D.__init__(self)
		# Chargement des images
		self.img = pygame.image.load("img.png")
		self.imgBoule_h = pygame.image.load("boule_h.png")
		self.imgBoule_h_i = pygame.image.load("boule_h_i.png")
		self.imgBoule_m = pygame.image.load("boule_m.png")
		self.imgBoule_s_i = pygame.image.load("boule_s_i.png")
		
		self.surface = surface    # Référence à la surface d'affichage
		self.focale = 300.0       # Profondeur du champ de vision
		self.dimEcran = Vecteur2D(300, 300)
		self.tabProf = []         # liste d'objets classés par profondeur
		self.temps = datetime.datetime(1, 1, 1)
		
		self.rot_x, self.rot_y, self.rot_z = 0, 1, 0  # Rotation autour des axes x, y, z
		self.mem_rot_x, self.mem_rot_y, self.mem_rot_z = 0, 1, 0
		self.stopperRotation = True
		
		# Construire un cercle avec 12 instances de Boule
		self.heures = []
		for i in range(12):
			x, y = 110 * math.sin( (math.pi/180.0)*(-i*30+180) ), 110 * math.cos( (math.pi/180.0)*(-i*30+180) )
			pos = Vecteur3D(x, y, 0)
			b = Boule(pos, 20.0, 0, self.focale, self.dimEcran, img=self.imgBoule_h, img_i=self.imgBoule_h_i)
			if i==0:
				b.texte = str(12)
			else:
				b.texte = str(i)
			self.heures.append(b)

		# Construire un cercle pour les minutes et les secondes
		self.minutes = []
		for i in range(60):
			x, y = 80 * math.sin( (math.pi/180.0)*(i*6+270) ), 80 * math.cos( (math.pi/180.0)*(i*6+270) )
			pos = Vecteur3D(0, x, y)
			b = Boule(pos, 12.0, 0, self.focale, self.dimEcran, img=self.imgBoule_m)
			b.tailleTexte = 20
			b.masquer = True
			if math.ceil(i/15.0) - math.floor(i/15.0) == 0:
				b.texte = str(i)
				b.masquer = False
			# Décaler les Boules des quarts
			if i == 15:
				b.pos.z += 10.0
			elif i == 30:
				b.pos.y += 10.0
			elif i == 45:
				b.pos.z -= 10.0
			elif i == 0:
				b.pos.y -= 10.0
			self.minutes.append(b)
		
		self.matrice = Matrice3D()    # Objet Matrice3D pour gérer la rotation
		self.verrouCommande = True

	def majLogique(self):
		""" Màj de la logique de l'horloge. """
		
		# Tester les événements
		for event in pygame.event.get():
			if event.type == pygame.QUIT:
				self.jeuActif = False
			elif event.type == pygame.KEYUP:
				self.verrouCommande = True
				if event.key == pygame.K_SPACE:
					if self.stopperRotation:
						# Mémoriser les valeurs de rotation x, y et z et stopper la rotation.
						self.stopperRotation = False
						self.mem_rot_x, self.mem_rot_y, self.mem_rot_z = self.rot_x, self.rot_y, self.rot_z
						self.rot_x, self.rot_y, self.rot_z = 0, 0, 0
					else:
						# Reprendre la rotation
						self.stopperRotation = True
						self.rot_x, self.rot_y, self.rot_z = self.mem_rot_x, self.mem_rot_y, self.mem_rot_z
				elif event.key == pygame.K_RETURN:
					# Réinitialiser les valeurs de rotation
					self.rot_x, self.rot_y, self.rot_z = 0, 0, 0
		
		# Récupérer les touches pressées
		touches = pygame.key.get_pressed()
		mod = 0
		modif = False
		if touches[pygame.K_LEFT]:
			mod -= 1
			modif = True
		elif touches[pygame.K_RIGHT]:
			mod += 1
			modif = True

		if touches[pygame.K_x] and modif:
			if self.verrouCommande:
				self.verrouCommande = False
				self.rot_x += mod
		elif touches[pygame.K_y] and modif:
			if self.verrouCommande:
				self.verrouCommande = False
				self.rot_y += mod
		elif touches[pygame.K_z] and modif:
			if self.verrouCommande:
				self.verrouCommande = False
				self.rot_z += mod
		# Récupérer le temps courant du système d'exploitation
		t = self.temps.now()
		if t.hour > 12:
			h = t.hour - 12
		else:
			h = t.hour
		m, s = t.minute, t.second

		# Màj des Boules des heures
		if h>0:
			self.heures[h-1].illuminer = False
			self.heures[h-1].couleurTexte = (255, 255, 255)
		else:
			self.heures[11].illuminer = False
			self.heures[11].couleurTexte = (255, 255, 255)
		self.heures[h].illuminer = True
		self.heures[h].couleurTexte = (0, 0, 0)

		# Màj des Boules des minutes
		for i in range(m+1):
			self.minutes[i].img = self.imgBoule_m
			self.minutes[i].masquer = False

		# Màj des Boules dees secondes
		if s>0:
			if s > m+1:
				if math.ceil((s-1)/15.0) - math.floor((s-1)/15.0) != 0:
					self.minutes[s-1].masquer = True
				self.minutes[s-1].img = self.imgBoule_m
		else:
			self.minutes[59].masquer = True
			if m == 0:
				# ràz des boules minutes
				iMin = 0
				for min in self.minutes:
					if math.ceil((iMin)/15.0) - math.floor((iMin)/15.0) != 0:
						min.masquer = True
					iMin += 1

		self.minutes[s].img = self.imgBoule_s_i
		self.minutes[s].masquer = False

		# Rotation de l'horloge
		mat = self.matrice.rotation(self.rot_x, self.rot_y, self.rot_z)
		for heure in self.heures:
			heure.pos = self.matrice.multiplierVecteur(heure.pos, mat)

		for minute in self.minutes:
			minute.pos = self.matrice.multiplierVecteur(minute.pos, mat)

		# Màj de  la profondeur des boules selon la distance z
		self.tabProf = []
		for heure in self.heures:
			self.tabProf.append([-heure.pos.z, heure])
		for minute in self.minutes:
			self.tabProf.append([-minute.pos.z, minute])
		self.tabProf.sort()
	
	def majAffichage(self):
		""" Màj de l'affichage de l'horloge. """
		self.surface.blit(self.img, (0, 0))		
		for e in self.tabProf:	
			e[1].afficher(self.surface)
		
		pygame.display.update()

def main():
	""" Utilisation :
			python horloge.py rot_init_h X Y Z rot_init_ms X Y Z rot X Y Z
			
				rot_init_h X Y Z : Rotation initiale du cadran des heures autour des axes X, Y et Z
				rot_init_ms X Y Z : Rotation initiale du cadran des minutes/secondes autour des axes X, Y et Z
				rot X Y Z : Rotation de l'horloge autour des axes X, Y et Z
		Exemples :
			python horloge.py rot_init_h 0 90 0
			python horloge.py rot_init_ms 90 45 0
			python horloge.py rot_init_h 0 90 0 rot 0 1 1
			python horloge.py rot_init_ms 0 90 90 rot 1 0 0
			python horloge.py rot 0 1 -1
			python horloge.py rot_init_h 0 90 0 rot_init_ms 0 60 120 rot 0 1 0
	"""
	pygame.init()
	surface = pygame.display.set_mode( (int(300), int(300)) )

	horloge = Horloge(surface)
	rot_init_h, rot_init_ms, rot = False, False, False
	nbrArgs = len(sys.argv)
	# Réglages en fonction des arguments
	if nbrArgs >= 5:
		if sys.argv[1] == 'rot_init_h':
			rot_init_h = True
			rot_init_h_x, rot_init_h_y, rot_init_h_z = int(sys.argv[2]), int(sys.argv[3]), int(sys.argv[4])
		if sys.argv[1] == 'rot_init_ms':
			rot_init_ms = True
			rot_init_ms_x, rot_init_ms_y, rot_init_ms_z = int(sys.argv[2]), int(sys.argv[3]), int(sys.argv[4])
		if sys.argv[1] == 'rot':
			rot = True
			rot_x, rot_y, rot_z = int(sys.argv[2]), int(sys.argv[3]), int(sys.argv[4])

		if nbrArgs >= 8 and nbrArgs < 13:
			if sys.argv[5] == 'rot':
				rot = True
				rot_x, rot_y, rot_z = int(sys.argv[6]), int(sys.argv[7]), int(sys.argv[8])

		if nbrArgs == 13:
			if sys.argv[5] == 'rot_init_ms':
				rot_init_ms = True
				rot_init_ms_x, rot_init_ms_y, rot_init_ms_z = int(sys.argv[6]), int(sys.argv[7]), int(sys.argv[8])
			if sys.argv[9] == 'rot':
				rot = True
				rot_x, rot_y, rot_z = int(sys.argv[10]), int(sys.argv[11]), int(sys.argv[12])
	
	if rot_init_h:
		mat = horloge.matrice.rotation(rot_init_h_x, rot_init_h_y, rot_init_h_z)
		for heure in horloge.heures:
			heure.pos = horloge.matrice.multiplierVecteur(heure.pos, mat)
	if rot_init_ms:
		mat = horloge.matrice.rotation(rot_init_ms_x, rot_init_ms_y, rot_init_ms_z)
		for minute in horloge.minutes:
			minute.pos = horloge.matrice.multiplierVecteur(minute.pos, mat)
	if rot:
		horloge.rot_x, horloge.rot_y, horloge.rot_z = rot_x, rot_y, rot_z

	horloge.maj()  # Lancer la boucle sans fin du programme

	
if __name__ == '__main__': 
	main()